from abc import ABCMeta, abstractmethod


class Template(metaclass=ABCMeta):

    def __init__(self):
        self._parts = []

    def put(self, value):
        self._parts.append(value)

    def format(self, **kwargs):
        delete = object()
        originals = {}
        for key in kwargs:
            if hasattr(self, key):
                originals[key] = getattr(self, key)
            else:
                originals[key] = delete
        try:
            for key, value in kwargs.items():
                setattr(self, key, value)
            self._parts.clear()
            self.run()
            return ''.join(self._parts)
        finally:
            for key, value in originals.items():
                if value is delete:
                    delattr(self, key)
                else:
                    setattr(self, key, value)

    @abstractmethod
    def run(self):
        pass


class ChainDict(dict):

    def __init__(self, *maps):
        # pylint: disable=super-init-not-called
        self._maps = maps

    def __missing__(self, key):
        for mapping in self._maps:
            try:
                return mapping[key]
            except KeyError:
                pass
        raise KeyError(key)



"""Templates -- Python templating library with templates included.

:copyright: (c) 2018 by Grant Jenks.
:license: Apache 2.0, see LICENSE for more details.

"""

from .base import Template
from .html import HTMLTemplate, Tag

__title__ = 'templates'
__version__ = '0.0.3'
__build__ = 0x000003
__author__ = 'Grant Jenks'
__license__ = 'Apache 2.0'
__copyright__ = '2018, Grant Jenks'



from .base import Template


class HTMLTemplate(Template):
    HTML_LANG = 'en'

    def run(self):
        self.doctype()
        self.html_element()

    def doctype(self):
        self.put('<!doctype html>')

    def html_element(self):
        with self.tag('html', {'lang': self.HTML_LANG}):
            self.head_element()
            self.body_element()

    def tag(self, name, attrs=None):
        return Tag(self, name, attrs or {})

    def head_element(self):
        with self.tag('head'):
            self.head()

    def head(self):
        pass

    def body_element(self):
        with self.tag('body'):
            self.body()

    def body(self):
        pass


class Tag:
    def __init__(self, template, name, attrs):
        self.template = template
        self.name = name
        self.attrs = attrs

    def __enter__(self):
        extra = ''
        attrs = self.attrs
        if attrs:
            items = attrs.items()
            pairs = ' '.join(f'{key}="{value}"' for key, value in items)
            extra = ' ' + pairs
        self.template.put(f'<{self.name}{extra}>')

    def __exit__(self, exc_type, exc_inst, exc_tb):
        self.template.put(f'</{self.name}>')
